#include "RTEngine.h"
#include "Image24.h"
#include "Primitive.h"
#include "PrimSphere.h"
#include <vector>

using namespace std;

RTEngine::RTEngine()
{
  return;
}

RTEngine::~RTEngine()
{
  vector<Primitive*>::iterator iter;

  for(iter = this->PrimitiveList.begin(); iter != this->PrimitiveList.end(); iter++)
    delete *iter;

  this->PrimitiveList.clear();
}

void RTEngine::AddSphere(Vector3D Position, double Radius)
{
  PrimSphere *prim = new PrimSphere(Position, Radius);
  this->PrimitiveList.push_back(prim);
}

void RTEngine::Render(Image24 *img)
{
  Vector3D CameraPos(0.0, 0.0, -5.0);
  double WX1 = -2.0, WX2 = 2.0, WY1 = 1.5, WY2 = -1.5;

  double DX = (WX2 - WX1) / (double)img->width;
  double DY = (WY2 - WY1) / (double)img->height;

  double SX = WX1, SY = WY1;

  int x, y;
  unsigned int *px = img->data;

  // rendering loop
  for(y = 0; y < img->height; y++)
  {
    SX = WX1;
    for(x = 0; x < img->width; x++)
    {
      Ray ray(CameraPos, Vector3D(SX, SY, 0.0));
      ray.Direction = ray.Direction - ray.Origin;
      ray.Direction.Norm();

      Vector3D color = this->Trace(ray);

      // double RGB -> unsigned int RGB
      int r, g, b;
      r = (int)(color.r * 256.0);
      g = (int)(color.g * 256.0);
      b = (int)(color.b * 256.0);

      r = r > 255 ? 255 : r < 0 ? 0 : r;
      g = g > 255 ? 255 : g < 0 ? 0 : g;
      b = b > 255 ? 255 : b < 0 ? 0 : b;

      *px = (r << 16) | (g << 8) | (b);

      // iter
      SX += DX;
      px++;
    }

    // iter
    SY += DY;
  }
}

// Trace
Vector3D RTEngine::Trace(Ray ray)
{
  Vector3D color(0.0, 0.0, 0.0);
  double dist = 1000000000.0f;
  Primitive *prim = NULL;

  // find closest intersection
  vector<Primitive*>::iterator iter;
  for(iter = this->PrimitiveList.begin(); iter != this->PrimitiveList.end(); iter++)
  {
    double temp_dist;
    if((*iter)->Intersect(ray, temp_dist) == 0)
      continue;

    if(temp_dist < dist)
    {
      prim = *iter;
      dist = temp_dist;
    }
  }

  if(prim)
    color = Vector3D(1.0, 1.0, 1.0);

  return color;
}


