#include <cstdio>
#include <cmath>
#include "Vector3D.h"

using namespace std;

Vector3D::Vector3D()
{
  this->x = this->y = this->z = 0.0;
}

Vector3D::Vector3D(double x, double y, double z)
{
  this->x = x;
  this->y = y;
  this->z = z;
}

void
Vector3D::PrintVector()
{
  printf("[%f, %f, %f]", this->x, this->y, this->z);
}

Vector3D&
Vector3D::operator=  (Vector3D v)
{
  this->x = v.x;
  this->y = v.y;
  this->z = v.z;
  return *this;
}

Vector3D&
Vector3D::operator-= (Vector3D v)
{
  this->x -= v.x;
  this->y -= v.y;
  this->z -= v.z;
  return *this;
}

Vector3D&
Vector3D::operator+= (Vector3D v)
{
  this->x += v.x;
  this->y += v.y;
  this->z += v.z;
  return *this;
}

Vector3D&
Vector3D::Set(double x, double y, double z )
{
  this->x = x;
  this->y = y;
  this->z = z;
  return *this;
}

Vector3D
Vector3D::operator+  (Vector3D v)
{
  Vector3D t;
  t.x = this->x + v.x;
  t.y = this->y + v.y;
  t.z = this->z + v.z;
  return t;
}

Vector3D
Vector3D::operator-  (Vector3D v)
{
  Vector3D t;
  t.x = this->x - v.x;
  t.y = this->y - v.y;
  t.z = this->z - v.z;
  return t;
}

Vector3D
Vector3D::operator*  (Vector3D v)
{
  Vector3D t;
  t.x = this->x * v.x;
  t.y = this->y * v.y;
  t.z = this->z * v.z;
  return t;
}

Vector3D
Vector3D::operator*= (Vector3D v)
{
  this->x *= v.x;
  this->y *= v.y;
  this->z *= v.z;
  return *this;
}

Vector3D
Vector3D::operator*  (double r)
{
  Vector3D t;
  t.x = this->x * r;
  t.y = this->y * r;
  t.z = this->z * r;
  return t;
}

Vector3D&
Vector3D::operator*= (double r)
{
  this->x *= r;
  this->y *= r;
  this->z *= r;
  return *this;
}

Vector3D
Vector3D::Cross(Vector3D v)
{
  Vector3D temp(
    this->y * v.z - this->z * v.y,
    this->z * v.x - this->x * v.z,
    this->x * v.y - this->y * v.x);
  return temp;
}

double
Vector3D::Dot(Vector3D v)
{
  return this->x * v.x + this->y * v.y + this->z * v.z;
}


double
Vector3D::Length()
{
  return sqrt(this->x * this->x + this->y * this->y + this->z * this->z);
}

double
Vector3D::SqrLength()
{
  return this->x * this->x + this->y * this->y + this->z * this->z;
}

void
Vector3D::Norm()
{
  double l = 1.0 / this->Length();
  this->x *= l;
  this->y *= l;
  this->z *= l;
}

